@@ -11,18 +11,15 @@ from sympy import (
     Expr,
     Float,
     Function,
-    Pow,
-    Wild,
-    log,
     simplify,
     sympify,
 )
-from sympy.codegen.cfunctions import log10
-from sympy.codegen.rewriting import ReplaceOptim, log1p_opt, optimize
 
 from chaste_codegen import LOGGER, CodegenError
 from chaste_codegen._math_functions import MATH_FUNC_SYMPY_MAPPING
+from chaste_codegen._optimize import optimize_expr_for_c_output
 from chaste_codegen._rdf import OXMETA, PYCMLMETA, get_variables_transitively
+from chaste_codegen._singularity_fixes import fix_singularity_equations
 
 
 MEMBRANE_VOLTAGE_INDEX = 0  # default index for voltage in state vector
@@ -35,31 +32,18 @@ STIM_PARAM_TAGS = (('membrane_stimulus_current_amplitude', 'uA_per_cm2', True),
                    ('membrane_stimulus_current_end', 'millisecond', False))
 
 
-# Optimisations to be applied to equations
-_V, _W = Wild('V'), Wild('W')
-# log(x)/log(10) --> log10(x)
-_LOG10_OPT = ReplaceOptim(_V * log(_W) / log(10), _V * log10(_W), cost_function=lambda expr: expr.count(
-    lambda e: (  # cost function prevents turning log(x) into log(10) * log10(x) as we want normal log in that case
-        e.is_Pow and e.exp.is_negative  # division
-        or (isinstance(e, (log, log10)) and not e.args[0].is_number))))
-# For P^n make sure n is passed as int if it is actually a whole number
-_POW_OPT = ReplaceOptim(lambda p: p.is_Pow and (isinstance(p.exp, Float) or isinstance(p.exp, float))
-                        and float(p.exp).is_integer(),
-                        lambda p: Pow(p.base, int(float(p.exp))))
-
-
-def load_model_with_conversions(model_file, use_modifiers=False, quiet=False):
+def load_model_with_conversions(model_file, use_modifiers=False, quiet=False, fix_singularities=True):
     if quiet:
         LOGGER.setLevel(logging.ERROR)
     try:
         model = cellmlmanip.load_model(model_file)
     except Exception as e:
         raise CodegenError('Could not load cellml model: \n    ' + str(e))
-    add_conversions(model, use_modifiers)
+    add_conversions(model, use_modifiers=use_modifiers, fix_singularities=fix_singularities)
     return model
 
 
-def add_conversions(model, use_modifiers=True):
+def add_conversions(model, use_modifiers=True, fix_singularities=True):
     # We are adding attributes to the model from cellmlmanip. This could break if the api changes
     # The check  below guards against this
     attrs_added = ('conversion_units', 'stimulus_units', 'time_variable', 'state_vars', 'membrane_voltage_var',
@@ -102,6 +86,12 @@ def add_conversions(model, use_modifiers=True):
 
     # Retrieve stimulus current parameters so we can exclude these from modifiers etc.
     model.modifiable_parameters = _get_modifiable_parameters(model)
+
+    # Fix singularities if desired
+    if fix_singularities:
+        fix_singularity_equations(model, model.membrane_voltage_var, model.modifiable_parameters)
+
+    # Get Capactiatnce & stimulus parameters
     model.membrane_capacitance = _get_membrane_capacitance(model)
     model.stimulus_params, model.stimulus_equations = _get_stimulus(model)
 
@@ -124,7 +114,7 @@ def add_conversions(model, use_modifiers=True):
     # Get derivs and eqs
     model.i_ionic_lhs, model.ionic_vars = _get_ionic_vars(model)
     model.extended_ionic_vars = _get_extended_ionic_vars(model)
-    model.derivative_equations = _get_derivative_equations(model)
+    model.derivative_equations = get_equations_for(model, model.y_derivatives)
 
 
 def set_is_metadata(model, variable, metadata_tag, ontology=PYCMLMETA):
@@ -156,13 +146,7 @@ def get_equations_for(model, variables, recurse=True, filter_modifiable_paramete
                  if not filter_modifiable_parameters_lhs or eq.lhs not in model.modifiable_parameters]
     if optimise:
         for i, eq in enumerate(equations):
-            optims = tuple()
-            if len(eq.rhs.atoms(log)) > 0:
-                optims += (_LOG10_OPT, log1p_opt)
-            if len(eq.rhs.atoms(Pow)) > 0:
-                optims += (_POW_OPT, )
-            if len(optims) > 0:
-                equations[i] = Eq(eq.lhs, optimize(eq.rhs, optims))
+            equations[i] = Eq(eq.lhs, optimize_expr_for_c_output(eq.rhs))
     return equations
 
 
@@ -519,9 +503,3 @@ def _get_extended_ionic_vars(model):
         raise CodegenError('Ionic variables should not be a function of time. '
                            'This is often caused by missing membrane_stimulus_current tag.')
     return extended_eqs
-
-
-def _get_derivative_equations(model):
-    """ Get equations defining the derivatives including V (model.membrane_voltage_var)"""
-    # Remove equations where lhs is a modifiable parameter or default stimulus
-    return [eq for eq in get_equations_for(model, model.y_derivatives)]
